<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:my="http://www.cdisc.org/ns/odm/v1.2"
	xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
<!-- Note that using version="2.0" causes an error when using Saxonb8-2 
	It complains that: "Error on line 293 of file:///C:/ODMViewer/ODM2ODMViewer.xsl:
  A sequence of more than one item is not allowed as the first argument of generate-id()" 
-->	

  <!-- Note that this stylesheet only works form ODM 1.2.0 -->

  <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
  
  <!-- 2.11.2014 whether Name + OID should be used for the tree node identifier and label -->
  <xsl:param name="USEOIDANDNAME">0</xsl:param>
  
  <xsl:key name="ItemGroupData-key" match="my:ItemGroupData" use="@ItemGroupOID"/>
    
  <xsl:template match="/">
  	<xsl:element name="ODMViewer">
  		<xsl:apply-templates select="ODM"/>
  		<xsl:apply-templates select="my:ODM"/>
  	</xsl:element>
  </xsl:template>
  
  <xsl:template match="ODM">
  	<xsl:message>NO NAMESPACE ODM IN ODM2ODMVIEWER.XSL</xsl:message>
  </xsl:template>
      
  <xsl:template match="my:ODM">
  	<xsl:message>TREATING ODM 1.2</xsl:message>
		<xsl:element name="Node">
			<xsl:attribute name="Label">ODM attributes</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/@*</xsl:attribute>
			<xsl:attribute name="Type">ODM</xsl:attribute>
		</xsl:element>
		<xsl:element name="Node">
      		<xsl:attribute name="Label">Studies</xsl:attribute>
      		<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study']</xsl:attribute>
      		<xsl:attribute name="Type">Study</xsl:attribute>
      		<xsl:apply-templates select="my:Study"/>
		</xsl:element>
		<!-- Admin Data node -->
		<xsl:element name="Node">
			<xsl:attribute name="Label">Administration data</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='AdminData']</xsl:attribute>
			<xsl:attribute name="Type">AdminData</xsl:attribute>
			<xsl:apply-templates select="my:AdminData"/>
		</xsl:element>
		<!-- Reference Data node -->
		<xsl:element name="Node">
			<xsl:attribute name="Label">Reference Data</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='ReferenceData']</xsl:attribute>
			<xsl:attribute name="Type">ReferenceData</xsl:attribute>
			<xsl:apply-templates select="my:ReferenceData"/>
		</xsl:element>
		<!-- Clinical Data node -->
		<xsl:element name="Node">
			<xsl:attribute name="Label">Clinical data</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='ClinicalData']</xsl:attribute>
			<xsl:attribute name="Type">ClinicalData</xsl:attribute>
			<xsl:apply-templates select="my:ClinicalData"/>
		</xsl:element>
		<!-- ds:Signature Node (if present) -->
		<xsl:if test="ds:Signature">
			<xsl:element name="Node">
				<xsl:attribute name="Label">ds:Signature</xsl:attribute>
				<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Signature']</xsl:attribute>
				<xsl:attribute name="Type">XML-Signature</xsl:attribute>
				<xsl:apply-templates select="ds:Signature"/>
			</xsl:element>
		</xsl:if>
		<!-- Subset Clinical Data Node -->
		<!--
		<xsl:element name="Node">
			<xsl:attribute name="Label">Clinical data subsets</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='ClinicalData']</xsl:attribute>
			<xsl:attribute name="Type">ClinicalDataSubsets</xsl:attribute>
			<xsl:apply-templates select="my:ClinicalData">
				<xsl:with-param name="subset">1</xsl:with-param>
			</xsl:apply-templates>
		</xsl:element> -->
  </xsl:template>
	
	<!-- Study OID -->
  	<xsl:template match="my:Study">
    	<xsl:element name="Node">
      		<xsl:variable name="OID"><xsl:value-of select="@OID"/></xsl:variable>
      		<xsl:attribute name="Label"><xsl:value-of select="@OID"/></xsl:attribute>
      		<xsl:attribute name="Type">Study</xsl:attribute>
      		<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study']/@OID='<xsl:value-of select="$OID"/>']</xsl:attribute>
      		<xsl:apply-templates select="my:GlobalVariables"/>
      		<xsl:apply-templates select="my:BasicDefinitions"/>
 
      		<xsl:element name="Node">
      			
      			<xsl:attribute name="Label">Metadata Versions</xsl:attribute>
      			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study']/@OID='<xsl:value-of select="$OID"/>/MetaDataVersion"/>']</xsl:attribute>
      			<xsl:attribute name="Type">MetaDataVersion</xsl:attribute>  
      			<xsl:apply-templates select="my:MetaDataVersion"/>
      	     </xsl:element> 
    	</xsl:element>
  	</xsl:template>
  	
  	<!-- Global Variables -->
  	<xsl:template match="my:GlobalVariables">
  		<xsl:variable name="STUDYOID" select="../@OID"/>
  		<xsl:variable name="APOS">'</xsl:variable>
  		<xsl:variable name="PATH" select="concat('/*[name()=', $APOS, 'ODM', $APOS, ']/*[name()=',$APOS, 'Study',$APOS, '][@OID=', $APOS, $STUDYOID, $APOS, ']/*[name()=', $APOS, 'GlobalVariables', $APOS, ']')"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Global variables</xsl:attribute>
  			<xsl:attribute name="XPath">
  				<xsl:value-of select="$PATH"/>
  			</xsl:attribute>
  			<xsl:attribute name="Type">Study</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- Basic Definitions -->
  	<xsl:template match="my:BasicDefinitions">
  		<xsl:variable name="STUDYOID" select="../@OID"/>
  		<xsl:variable name="APOS">'</xsl:variable>
  		<xsl:variable name="PATH" select="concat('/*[name()=', $APOS, 'ODM', $APOS, ']/*[name()=',$APOS, 'Study',$APOS, '][@OID=', $APOS, $STUDYOID, $APOS, ']/*[name()=', $APOS, 'BasicDefinitions', $APOS, ']')"/>
  		<xsl:variable name="PATH2" select="concat($PATH, '/*[name()=', $APOS, 'MeasurementUnit', $APOS, ']')"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Basic definitions</xsl:attribute>
  			<!--xsl:attribute name="XPath">/ODM/Study[@OID='<xsl:value-of select="$STUDYOID"/>']/BasicDefinitions</xsl:attribute-->
  			<xsl:attribute name="XPath"><xsl:value-of select="$PATH"/></xsl:attribute>
  			<xsl:attribute name="Type">Study</xsl:attribute>
  			<xsl:element name="Node">
  				<xsl:attribute name="Label">Units of Measurements</xsl:attribute>
  				<!--xsl:attribute name="XPath">/ODM/Study[@OID='<xsl:value-of select="$STUDYOID"/>']/BasicDefinitions/MeasurementUnit</xsl:attribute-->
  				<xsl:attribute name="XPath"><xsl:value-of select="$PATH2"/></xsl:attribute>
  				<xsl:attribute name="Type">MeasurementsUnits</xsl:attribute>
  			</xsl:element>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- MetaDataVersion -->
  	<xsl:template match="my:MetaDataVersion">
  		<xsl:variable name="STUDYOID" select="../@OID"/>
  		<xsl:variable name="NAME" select="@Name"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label"><xsl:value-of select="@Name"/></xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion']</xsl:attribute>
  			<xsl:attribute name="Type">Study</xsl:attribute>
  			<xsl:apply-templates select="my:Include[1]"/>
  			<xsl:apply-templates select="my:Protocol[1]"/>
  			<xsl:apply-templates select="my:StudyEventDef[1]"/>
  			<xsl:apply-templates select="my:FormDef[1]"/>
  			<xsl:apply-templates select="my:ItemGroupDef[1]"/>
  			<xsl:apply-templates select="my:ItemDef[1]"/>
  			<xsl:apply-templates select="my:CodeList[1]"/>
  			<xsl:apply-templates select="my:ImputationMethod[1]"/>
  			<xsl:apply-templates select="my:Presentation[1]"/>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- Include: 0 or 1 occurences-->
  	<xsl:template match="my:Include">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Include</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='Include']</xsl:attribute>
  			<xsl:attribute name="Type">Include</xsl:attribute>
  		</xsl:element>
  	</xsl:template>	
  	
  	<!-- Protocol: 0 or 1 occurences -->
  	<xsl:template match="my:Protocol">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Protocol</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='Protocol']</xsl:attribute>
  			<xsl:attribute name="Type">Protocol</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- StudyEventDef: 0 or more occurrences -->
  	<xsl:template match="my:StudyEventDef">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Study events</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='StudyEventDef']</xsl:attribute>
  			<xsl:attribute name="Type">StudyEventDef</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- FormDef: 0 or more occurrences -->
  	<xsl:template match="my:FormDef">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Forms</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='FormDef']</xsl:attribute>
  			<xsl:attribute name="Type">FormDef</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- ItemDef: 0 or more occurrences -->
  	<xsl:template match="my:ItemGroupDef">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Item groups</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='ItemGroupDef']</xsl:attribute>
  			<xsl:attribute name="Type">ItemGroupDef</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- ItemDef: 0 or more occurrences -->
  	<xsl:template match="my:ItemDef">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Items</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='ItemDef']</xsl:attribute>
  			<xsl:attribute name="Type">ItemDef</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- CodeList: 0 or more occurrences -->
  	<xsl:template match="my:CodeList">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Codelists</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='CodeList']</xsl:attribute>
  			<xsl:attribute name="Type">CodeList</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- ImputationMethod: 0 or more occurrences -->
  	<xsl:template match="my:ImputationMethod">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Imputation methods</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='ImputationMethod']</xsl:attribute>
  			<xsl:attribute name="Type">ImputationMethod</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
  	
  	<!-- Presentation -->
  	<xsl:template match="my:Presentation">
  		<xsl:variable name="VERSION" select="../@OID"/>
  		<xsl:variable name="STUDYOID" select="../../@OID"/>
  		<xsl:element name="Node">
  			<xsl:attribute name="Label">Presentations</xsl:attribute>
  			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Study'][@OID='<xsl:value-of select="$STUDYOID"/>']/*[name()='MetaDataVersion'][@OID='<xsl:value-of select="$VERSION"/>']/*[name()='Presentation']</xsl:attribute>
  			<xsl:attribute name="Type">Presentation</xsl:attribute>
  		</xsl:element>
  	</xsl:template>
	
	<xsl:template match="my:AdminData">
		<xsl:element name="Node">
			<xsl:attribute name="Label"><xsl:value-of select="@StudyOID"/></xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='AdminData'][@StudyOID='<xsl:value-of select="@StudyOID"/>']</xsl:attribute>
			<xsl:attribute name="Type">AdminData</xsl:attribute>
			<xsl:apply-templates select="my:User[1]"/>
			<xsl:apply-templates select="my:Location[1]"/>
			<xsl:apply-templates select="my:SignatureDef[1]"/>
		</xsl:element>
	</xsl:template>
	
	<!-- User data -->
	<xsl:template match="my:User">
		<xsl:element name="Node">
			<xsl:attribute name="Label">Users</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='AdminData'][@StudyOID='<xsl:value-of select="../@StudyOID"/>']/*[name()='User']</xsl:attribute>
			<xsl:attribute name="Type">User</xsl:attribute>
		</xsl:element>
	</xsl:template>
	
	<!-- Location data -->
	<xsl:template match="my:Location">
		<xsl:element name="Node">
			<xsl:attribute name="Label">Locations</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='AdminData'][@StudyOID='<xsl:value-of select="../@StudyOID"/>']/*[name()='Location']</xsl:attribute>
			<xsl:attribute name="Type">Location</xsl:attribute>
		</xsl:element>
	</xsl:template>
	
	<!-- Signature Definitions -->
	<xsl:template match="my:SignatureDef">
		<xsl:element name="Node">
			<xsl:attribute name="Label">Signature defs</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='AdminData'][@StudyOID='<xsl:value-of select="../@StudyOID"/>']/*[name()='SignatureDef']</xsl:attribute>
			<xsl:attribute name="Type">SignatureDef</xsl:attribute>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="my:ReferenceData">
		<xsl:element name="Node">
			<xsl:attribute name="Label"><xsl:value-of select="@StudyOID"/></xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='ReferenceData'][@StudyOID='<xsl:value-of select="@StudyOID"/>']</xsl:attribute>
			<xsl:attribute name="Type">ReferenceData</xsl:attribute>
			
			<xsl:element name="Node">
				<xsl:attribute name="Label"><xsl:value-of select="@MetaDataVersionOID"/></xsl:attribute>
				<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='ReferenceData'][@StudyOID='<xsl:value-of select="@StudyOID"/>'][@MetaDataVersionOID='<xsl:value-of select="@MetaDataVersionOID"/>']</xsl:attribute>
				<!-- we now need to get all unique item groups -->
				<!--xsl:variable name="unique-ItemGroupData" 
			  		select="my:ItemGroupData[generate-id(.) = generate-id(key('ItemGroupData-key',@ItemGroupOID))]/@ItemGroupOID"></xsl:variable-->
			  	<xsl:variable name="unique-ItemGroupData" 
			  		select="my:ItemGroupData[generate-id(.) = generate-id(key('ItemGroupData-key',@ItemGroupOID))]/@ItemGroupOID"></xsl:variable>
				<xsl:variable name="STUDYOID"><xsl:value-of select="@StudyOID"/></xsl:variable>
				<xsl:variable name="VERSION"><xsl:value-of select="@MetaDataVersionOID"/></xsl:variable>
				<xsl:variable name="APOS">'</xsl:variable> 
				<!-- loop over all unique ItemGroups -->
				<xsl:for-each select="$unique-ItemGroupData">
					<xsl:variable name="ITEMGROUPOID"><xsl:value-of select="."/></xsl:variable>
					<!--xsl:variable name="LABEL" select="/my:ODM/my:Study[@OID=$STUDYOID]/my:MetaDataVersion[@OID=$VERSION]/my:ItemGroupDef[@OID=$ITEMGROUPOID]/@Name"/-->
					<!-- 3.3.2011 -->
					<xsl:variable name="LABEL">
						<xsl:call-template name="GetNameOfDefElement">
							<xsl:with-param name="studyoid" select="$STUDYOID"/>
							<xsl:with-param name="metadataversionoid" select="$VERSION"/>
							<xsl:with-param name="elementname">ItemGroupDef</xsl:with-param>
							<xsl:with-param name="elementoid" select="$ITEMGROUPOID"/>
						</xsl:call-template>
					</xsl:variable>
					<!--xsl:variable name="PATH" select="concat('/ODM/ReferenceData[@StudyOID=',$APOS,$STUDYOID,$APOS,'][MetaDataVersionOID=', 
					$APOS,$VERSION,$APOS,']/ItemGroupData[@ItemGroupOID=',$APOS,$ITEMGROUPOID,$APOS,']')"/-->
					<xsl:variable name="PATH">/*[name()='ODM']/*[name()='ReferenceData'][@StudyOID='<xsl:value-of select="$STUDYOID"/>'][@MetaDataVersionOID='<xsl:value-of select="$VERSION"/>']/*[name()='ItemGroupData'][@ItemGroupOID='<xsl:value-of select="$ITEMGROUPOID"/>']</xsl:variable>
	            	<!-- construct a new subelement -->
					<xsl:element name="Node">
						<xsl:choose> <!-- it sometimes happens that the ItemGroup was NOT defined in the Study section -->
						<!-- ItemGroup WAS defined in the Study section -->
							<xsl:when test="$LABEL != ''">
								<xsl:attribute name="Label"><xsl:value-of select="$LABEL"/></xsl:attribute>
							</xsl:when>
							<!-- ItemGroup was NOT defined in the Study section, just take the ItemGroupOID as label -->
							<xsl:otherwise>
								<xsl:attribute name="Label"><xsl:value-of select="$ITEMGROUPOID"/></xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:attribute name="XPath"><xsl:value-of select="$PATH"/></xsl:attribute>
						<xsl:attribute name="Type">ReferenceData</xsl:attribute>
					</xsl:element> 
				</xsl:for-each>
			</xsl:element>			
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="my:ClinicalData">
		<!-- the 'subset' parameter determines whether it goes about subset data (1) or regular data (0) -->
		<xsl:param name="subset">0</xsl:param>
    	<xsl:element name="Node">
      	<xsl:variable name="OID"><xsl:value-of select="@StudyOID"/></xsl:variable>
      	<xsl:variable name="VERSION"><xsl:value-of select="@MetaDataVersionOID"/></xsl:variable>
      	<xsl:attribute name="Label"><xsl:value-of select="@StudyOID"/></xsl:attribute>
      	<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='ClinicalData'][@StudyOID='<xsl:value-of select="$OID"/>']</xsl:attribute>
      	<xsl:choose>
      		<xsl:when test="$subset=0">
      			<xsl:attribute name="Type">ClinicalData</xsl:attribute>
      		</xsl:when>
      		<xsl:otherwise>
      			<xsl:attribute name="Type">ClinicalDataSubset</xsl:attribute>
      		</xsl:otherwise>
      	</xsl:choose>
      	
      	<xsl:element name="Node">
        	<xsl:attribute name="Label"><xsl:value-of select="$VERSION"/></xsl:attribute>
        	<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='ClinicalData'][@StudyOID='<xsl:value-of select="$OID"/>'][@MetaDataVersionOID='<xsl:value-of select="$VERSION"/>']</xsl:attribute>
        	<xsl:choose>
        		<xsl:when test="$subset=0">
      				<xsl:attribute name="Type">ClinicalData</xsl:attribute>
      			</xsl:when>
      			<xsl:otherwise>
      				<xsl:attribute name="Type">ClinicalDataSubset</xsl:attribute>
      			</xsl:otherwise>
      		</xsl:choose>
        	<!-- item groups, if not subset data -->
        	<xsl:if test="$subset=0">
        		<xsl:apply-templates select="my:SubjectData"/>
        	</xsl:if>
      	</xsl:element>
    </xsl:element>
	</xsl:template>
	
	<xsl:template match="my:SubjectData">
		<!-- the 'subset' parameter determines whether it goes about subset data (1) or regular data (0) -->
    	<xsl:apply-templates select="my:StudyEventData"><!-- do nothing special -->
    	</xsl:apply-templates> 
	</xsl:template>
	
	<xsl:template match="my:StudyEventData">
    	<xsl:apply-templates select="my:FormData"></xsl:apply-templates>
	</xsl:template>
	
	<xsl:template match="my:FormData">
    	<!-- create unique ItemGroups -->
    	<xsl:variable name="unique-ItemGroupData" 
			  select="my:ItemGroupData[generate-id(.) = generate-id(key('ItemGroupData-key',@ItemGroupOID))]/@ItemGroupOID"></xsl:variable>
		<xsl:variable name="STUDYOID"><xsl:value-of select="../../../@StudyOID"/></xsl:variable>
		<xsl:variable name="VERSION"><xsl:value-of select="../../../@MetaDataVersionOID"/></xsl:variable>
		<xsl:variable name="APOS">'</xsl:variable>
		
		<xsl:for-each select="$unique-ItemGroupData">
			<!-- determine the name of the ItemGroup -->
			<xsl:variable name="ITEMGROUPOID"><xsl:value-of select="."/></xsl:variable>
			<xsl:message>Treating ITEMGROUPOID = <xsl:value-of select="$ITEMGROUPOID"/></xsl:message>
			<!--xsl:variable name="LABEL" select="/my:ODM/my:Study[@OID=$STUDYOID]/my:MetaDataVersion[@OID=$VERSION]/my:ItemGroupDef[@OID=$ITEMGROUPOID]/@Name"/-->
			<!-- 3.3.2011 -->
			<xsl:variable name="LABEL">
				<xsl:call-template name="GetNameOfDefElement">
					<xsl:with-param name="studyoid" select="$STUDYOID"/>
					<xsl:with-param name="metadataversionoid" select="$VERSION"/>
					<xsl:with-param name="elementname">ItemGroupDef</xsl:with-param>
					<xsl:with-param name="elementoid" select="$ITEMGROUPOID"/>
				</xsl:call-template>
			</xsl:variable>
			<!--xsl:variable name="PATH" select="concat('/ODM/ClinicalData[@StudyOID=',$APOS,$STUDYOID,$APOS,'][MetaDataVersionOID=', 
			$APOS,$VERSION,$APOS,']/SubjectData/StudyEventData/ItemGroupData[@ItemGroupOID=',$APOS,$ITEMGROUPOID,$APOS,']')"/-->
			<xsl:variable name="PATH">/*[name()='ODM']/*[name()='ClinicalData'][@StudyOID='<xsl:value-of select="$STUDYOID"/>'][@MetaDataVersionOID='<xsl:value-of select="$VERSION"/>']/*[name()='SubjectData']/*[name()='StudyEventData']/*[name()='FormData']/*[name()='ItemGroupData'][@ItemGroupOID='<xsl:value-of select="$ITEMGROUPOID"/>']</xsl:variable>
			<xsl:element name="Node">
				<!-- In some cases only the name of the ItemGroup was not defined, but only the OID -->
				<xsl:choose>
					<xsl:when test="$LABEL != ''">
						<!-- <xsl:attribute name="Label"><xsl:value-of select="$LABEL"/> - <xsl:value-of select="."/></xsl:attribute> until 10.10.2014 -->
						<!-- change 10.10.2014 - use "Name" + "OID" -->
						<!-- change 2.11.2014 - use parameter from software -->
						<xsl:choose>
							<xsl:when test="$USEOIDANDNAME = '1'">
								<xsl:attribute name="Label"><xsl:value-of select="$LABEL"/> - <xsl:value-of select="."/></xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="Label"><xsl:value-of select="$LABEL"/></xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:attribute name="Label"><xsl:value-of select="."/></xsl:attribute>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:attribute name="XPath"><xsl:value-of select="$PATH"/></xsl:attribute>
				<xsl:attribute name="Type">ClinicalData</xsl:attribute>
			</xsl:element>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="ds:Signature">
		<!-- We make three nodes, one for SignedInfo,
			one for SignatureValue, and one for KeyInfo (the latter is optional) -->
		<xsl:element name="Node">
			<xsl:attribute name="Label">SignedInfo</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Signature']/*[name()='SignedInfo']</xsl:attribute>
			<xsl:attribute name="Type">SignedInfo</xsl:attribute>
		</xsl:element>
		<xsl:element name="Node">
			<xsl:attribute name="Label">SignatureValue</xsl:attribute>
			<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Signature']/*[name()='SignatureValue']</xsl:attribute>
			<xsl:attribute name="Type">SignatureValue</xsl:attribute>
		</xsl:element>
		<xsl:if test="ds:KeyInfo">
			<xsl:element name="Node">
				<xsl:attribute name="Label">KeyInfo</xsl:attribute>
				<xsl:attribute name="XPath">/*[name()='ODM']/*[name()='Signature']/*[name()='KeyInfo']</xsl:attribute>
				<xsl:attribute name="Type">KeyInfo</xsl:attribute>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<!-- 3.3.2011 -->
	<xsl:template name="GetNameOfDefElement">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<xsl:param name="elementname"/>
		<xsl:param name="elementoid"/>
		<xsl:choose>
			<!-- first case: XXXDef in the same MetaDataVersion -->
			<xsl:when test="/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()=$elementname][@OID=$elementoid]">
				<xsl:value-of select="/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()=$elementname][@OID=$elementoid]/@Name"/>
			</xsl:when>
			<!-- second case: Not in the same MetaDataVersion, but there is an Include -->
			<!-- TODO: make this recursive -->
			<xsl:when test="/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']">
				<xsl:variable name="INCLUDESTUDYOID" select="/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@StudyOID"/>
				<xsl:variable name="INCLUDEMETADATAVERSIONOID" select="/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@MetaDataVersionOID"/>
				<xsl:message>Trying to find Name for element <xsl:value-of select="$elementname"/> with OID <xsl:value-of select="$elementoid"/> through Include, $INCLUDESTUDYOID = <xsl:value-of select="$INCLUDESTUDYOID"/> - $INCLUDEMETADATAVERSIONOID = <xsl:value-of select="$INCLUDEMETADATAVERSIONOID"/></xsl:message>
				<!-- implements recursion -->
				<xsl:variable name="NAMEVALUE">
					<xsl:choose>
						<xsl:when test="/*[name()='ODM']/*[name()='Study'][@OID=$INCLUDESTUDYOID]/*[name()='MetaDataVersion'][@OID=$INCLUDEMETADATAVERSIONOID]/*[name()=$elementname][@OID=$elementoid]">
							<xsl:value-of select="/*[name()='ODM']/*[name()='Study'][@OID=$INCLUDESTUDYOID]/*[name()='MetaDataVersion'][@OID=$INCLUDEMETADATAVERSIONOID]/*[name()=$elementname][@OID=$elementoid]/@Name"/>
						</xsl:when>
						<!-- nothing found in the to-be-included MetaDataVersion, try recursion -->
						<xsl:otherwise>
							<xsl:call-template name="GetNameOfDefElement">
								<xsl:with-param name="metadataversionoid" select="$INCLUDEMETADATAVERSIONOID"/>
								<xsl:with-param name="studyoid" select="$INCLUDESTUDYOID"/>
								<xsl:with-param name="elementname" select="$elementname"/>
								<xsl:with-param name="elementoid" select="$elementoid"/>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:value-of select="$NAMEVALUE"/>
			</xsl:when>
			<!-- no Include and not in this MetaDataVersion, return an empty name -->
			<xsl:otherwise></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
  
</xsl:stylesheet>
