<?xml version="1.0" encoding="UTF-8"?>

<!-- Stylesheet merging metadata and clinical data from two documents into a single documenr -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:ds="http://www.w3.org/2000/09/xmldsig#" version="1.1">
	
	<!--xsl:param name="METADATAFILE">CES_StudyDesign_2006_01_26_extra_languages.xml</xsl:param-->
	<xsl:param name="METADATAFILE"></xsl:param>
	<xsl:output method="xml" indent="yes"/>
	
	<!-- start template -->
	<xsl:template match="/">
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="*[name()='ODM']">
		<xsl:copy><!-- copy the ODM element itself -->
			<xsl:copy-of select="@*"/>
			<xsl:message>Starting merging with file = <xsl:value-of select="$METADATAFILE"/></xsl:message>
			<!-- first look for the MetaData (Element Study) in the file with metadata -->
			<!--   <xsl:for-each select="$METADATAFILE/*[name()='ODM']/*[name()='Study']/*[name()='MetaDataVersion']">  -->
			<xsl:for-each select="document($METADATAFILE)/*[name()='ODM']/*[name()='Study']">
				<xsl:message>Found Study with OID = <xsl:value-of select="@OID"/></xsl:message>
				<xsl:message>Copying Study into file with ClinicalData</xsl:message>
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<!-- if ReferenceData are found in the file with ClinicalData, copy them -->
			<xsl:choose>
				<xsl:when test="./*[name()='ReferenceData']">
					<xsl:message>Copying ReferenceData from file with ClinicalData - ReferenceData from file with MetaData will be ignored</xsl:message>
					<xsl:copy-of select="./*[name()='ReferenceData']"/>
				</xsl:when>
				<!-- alternatively, copy from the file with MetaData -->
				<xsl:when test="document($METADATAFILE)//*[name()='ReferenceData']">
					<xsl:message>Copying ReferenceData from file with MetaData</xsl:message>
					<xsl:copy-of select="document($METADATAFILE)//*[name()='ReferenceData']"/>
				</xsl:when>
				<xsl:otherwise><xsl:message>No ReferenceData were found, neither in file with ClinicalData, nor in file with MetaData</xsl:message></xsl:otherwise>
			</xsl:choose>
			<!-- if AdminData are found in the file with ClinicalData, copy them -->
			<xsl:choose>
				<xsl:when test="./*[name()='AdminData']">
					<xsl:message>Copying AdminData from file with ClinicalData - AdminData from file with MetaData will be ignored</xsl:message>
					<xsl:copy-of select="./*[name()='AdminData']"/>
				</xsl:when>
				<!-- alternatively, copy from the file with MetaData -->
				<xsl:when test="document($METADATAFILE)//*[name()='AdminData']">
					<xsl:message>Copying AdminData from file with MetaData</xsl:message>
					<xsl:copy-of select="document($METADATAFILE)//*[name()='AdminData']"/>
				</xsl:when>
				<xsl:otherwise><xsl:message>No AdminData were found, neither in file with ClinicalData, nor in file with MetaData</xsl:message></xsl:otherwise>
			</xsl:choose>
			<!-- copy all clinical data from source file -->
			<xsl:for-each select="./*[name()='ClinicalData']">
				<xsl:message>Copying ClinicalData with StudyOID = <xsl:value-of select="@StudyOID"/> and MetaDataVersionOID = <xsl:value-of select="@MetaDataVersionOID"/></xsl:message>
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<!-- copy all association data from the source file -->
			<xsl:for-each select="./*[name()='Association']">
				<xsl:message>Copying Association data</xsl:message>
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<xsl:for-each select="ds:Signature">
				<xsl:message>Copying ds:Signature data</xsl:message>
				<xsl:copy-of select="."/>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
	
</xsl:stylesheet>