<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
	
	<xsl:param name="ODMDOC"></xsl:param>
	<!-- P.S. Cannot apply coloring here by adding <font ..> -->
	<!--xsl:variable name="CODELISTERROR">NOT IN CODELIST !</xsl:variable--><!-- message for the case an Item Value could not be decoded -->
	<xsl:variable name="CODELISTERROR">(!NOT IN CODELIST!)</xsl:variable><!-- Behring 8.7.2014 message for the case an Item Value could not be decoded -->
	<xsl:template match="//Results">
		<xsl:variable name="METADATAVERSIONOID" select="@MetaDataVersionOID"/>
		<xsl:variable name="STUDYOID" select="@StudyOID"/>
		<xsl:choose>
		<xsl:when test="@Type='Protocol'">
			<!-- Protocols -->
			<xsl:copy>
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:apply-templates select="Protocol">
					<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
					<xsl:with-param name="studyoid" select="$STUDYOID"/>
				</xsl:apply-templates>
			</xsl:copy>
		</xsl:when>	
		
		<xsl:when test="@Type='User'">
			<xsl:copy>  <!-- copy the Results element and its attributes -->
				<xsl:copy-of select="@*"/>
				<xsl:for-each select="User">
					<xsl:copy>  <!-- copy the User element and its attributes -->
						<xsl:copy-of select="@*"/>
						<!-- make deep copies of elements (also OK if >1 occurence) -->
						<xsl:copy-of select="LoginName"/>
						<xsl:copy-of select="DisplayName"/>
						<xsl:copy-of select="FullName"/>
						<xsl:copy-of select="FirstName"/>
						<xsl:copy-of select="LastName"/>
						<xsl:copy-of select="Organization"/>
						<xsl:copy-of select="Address"/>
						<xsl:copy-of select="Email"/>
						<xsl:copy-of select="Picture"/>
						<xsl:copy-of select="Pager"/>
						<xsl:copy-of select="Fax"/>
						<xsl:copy-of select="Phone"/>
						<!-- add information about the Location -->
						<xsl:apply-templates select="LocationRef">
							<xsl:with-param name="studyoid" select="$STUDYOID"/>
							<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
						</xsl:apply-templates>
						<xsl:copy-of select="Certificate"/>
					</xsl:copy>
				</xsl:for-each>
			</xsl:copy>
		</xsl:when>
		
		
		<!-- Location, we need to add the MetaDataVersion name -->
		<xsl:when test="@Type='Location'">
			<xsl:copy>
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>	<!-- copy the attributes -->
				<xsl:for-each select="Location">
					<xsl:copy>
						<xsl:for-each select="@*">
							<xsl:copy/>
						</xsl:for-each>
						<xsl:apply-templates select="MetaDataVersionRef"/>
					</xsl:copy>
				</xsl:for-each>
			</xsl:copy>
		
		</xsl:when>
		<!-- StudyEvent's : we need to get Form Name from Form OID -->
		<xsl:when test="@Type='StudyEventDef'">
			<xsl:copy>
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:apply-templates select="StudyEventDef">
					<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
					<xsl:with-param name="studyoid" select="$STUDYOID"/>
				</xsl:apply-templates>
			</xsl:copy>
		</xsl:when>
		<!-- Forms: we need to get ItemGroup Name from ItemGroup OID -->
		<xsl:when test="@Type='FormDef'">
			<xsl:copy>
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:apply-templates select="FormDef">
					<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
					<xsl:with-param name="studyoid" select="$STUDYOID"/>
				</xsl:apply-templates>
			</xsl:copy>
		</xsl:when>
		<xsl:when test="@Type='ItemGroupDef'">
			<xsl:copy>
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:apply-templates select="ItemGroupDef">
					<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
					<xsl:with-param name="studyoid" select="$STUDYOID"/>
				</xsl:apply-templates>
			</xsl:copy>
		</xsl:when>
		
		<!-- ItemDef -->
		<xsl:when test="@Type='ItemDef'">
			<xsl:copy> <!-- copy the Results element and its attributes -->
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:for-each select="ItemDef">
					<xsl:copy>  <!-- copy all ItemDef elements and its attributes -->
						<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
						<!-- For ODM 1.3, copy the contents of the Description element -->
						<xsl:for-each select="Description">
							<xsl:copy-of select="."/>
						</xsl:for-each>
						<xsl:copy-of select="Question"/>
						<xsl:copy-of select="ExternalQuestion"/>
						<xsl:apply-templates select="MeasurementUnitRef">
							<xsl:with-param name="studyoid" select="$STUDYOID"/>
							<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
						</xsl:apply-templates>
						<xsl:copy-of select="RangeCheck"/>
						<xsl:apply-templates select="CodeListRef">
							<xsl:with-param name="studyoid" select="$STUDYOID"/>
							<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
						</xsl:apply-templates>
						<xsl:copy-of select="Role"/>
						<xsl:copy-of select="Alias"/>
					</xsl:copy>
				</xsl:for-each>
			</xsl:copy>
		</xsl:when>
		<!-- ClinicalData and ReferenceData -->
		<xsl:when test="@Type='ClinicalData' or @Type='ReferenceData'">
			<xsl:copy>
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<!-- If there were any selections (subset) copy them -->
				<xsl:for-each select="Selections">
					<!--xsl:message>trying to copy Selections element</xsl:message-->
					<xsl:copy-of select="."/>
				</xsl:for-each>
				<xsl:apply-templates select="ItemGroupData">
					<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
					<xsl:with-param name="studyoid" select="$STUDYOID"/>
				</xsl:apply-templates>
			</xsl:copy>
		</xsl:when>  
		<xsl:when test="@Type='AuditRecords'">
			<xsl:copy>
				<!-- copy the attributes -->
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:apply-templates select="AuditRecord"> 
						<xsl:with-param name="studyoid" select="$STUDYOID"/>
						<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
				</xsl:apply-templates> 
			</xsl:copy>
		</xsl:when>
		
		<xsl:when test="@Type='Signatures'">
			<!--xsl:message>TYPE = Signatures</xsl:message-->
			<xsl:copy>
				<!-- copy the attributes -->
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:apply-templates select="Signature">
					<xsl:with-param name="studyoid" select="$STUDYOID"/>
					<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
				</xsl:apply-templates>
				</xsl:copy>
		</xsl:when>
		
		<!-- 18.6.2009 -->
		<xsl:when test="@Type='Annotations'">
			<xsl:copy>
				<!-- copy the attributes -->
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<!-- can only have Annotation elements -->
				<xsl:apply-templates/>
			</xsl:copy>
		</xsl:when>
		
		<!-- 8.6.2009 AuditRecords Subsets -->
		<xsl:when test="@Type='AuditRecordsSubset'">
			<xsl:copy>
				<!-- copy the attributes -->
				<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				<xsl:for-each select="Selections">
					<!--xsl:message>trying to copy Selections element</xsl:message-->
					<xsl:copy-of select="."/>
				</xsl:for-each>
				<xsl:for-each select="AuditRecord">
					<xsl:variable name="STUDYOID" select="@StudyOID"/>
					<xsl:variable name="METADATAVERSIONOID" select="@MetaDataVersionOID"/>
					<xsl:copy>
						<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
						<!-- find the NAMES -->
						<!-- 7.3.2011 MetaDataVersionName -->
						<xsl:attribute name="MetaDataVersionName" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/@Name"/>
						<!-- The Investigator Name -->
						<xsl:variable name="INVESTIGATOROID" select="@InvestigatorOID"/>
						<xsl:variable name="INVESTIGATORNAME">
							<xsl:choose>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='User'][@OID=$INVESTIGATOROID]/*[name()='FullName'] != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='User'][@OID=$INVESTIGATOROID]/*[name()='FullName']"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$INVESTIGATOROID"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:attribute name="InvestigatorName"><xsl:value-of select="$INVESTIGATORNAME"/></xsl:attribute>
						<!-- Site name -->
						<xsl:variable name="SITEOID" select="@SiteOID"/>
						<xsl:variable name="SITENAME">
							<xsl:choose>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='Location'][@OID=$SITEOID]/@Name != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='Location'][@OID=$SITEOID]/@Name"/>
								</xsl:when>
								<xsl:otherwise><xsl:value-of select="$SITEOID"/></xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:attribute name="SiteName"><xsl:value-of select="$SITENAME"/></xsl:attribute>
						<!-- StudyEvent name -->
						<xsl:variable name="STUDYEVENTOID" select="@StudyEventOID"/>
						<!--xsl:variable name="STUDYEVENTNAME">
							<xsl:choose>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='StudyEventDef'][@OID=$STUDYEVENTOID]/@Name != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='StudyEventDef'][@OID=$STUDYEVENTOID]/@Name"/>
								</xsl:when>
								<xsl:otherwise><xsl:value-of select="$STUDYEVENTOID"/></xsl:otherwise>
							</xsl:choose>
						</xsl:variable-->
						<!-- 3.3.2011 -->
						<xsl:variable name="STUDYEVENTNAME">
							<xsl:call-template name="GetNameOfDefElement">
								<xsl:with-param name="studyoid" select="$STUDYOID"/>
								<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
								<xsl:with-param name="elementname">StudyEventDef</xsl:with-param>
								<xsl:with-param name="elementoid" select="$STUDYEVENTOID"/>
							</xsl:call-template>
						</xsl:variable>
						<xsl:attribute name="StudyEventName"><xsl:value-of select="$STUDYEVENTNAME"/></xsl:attribute>
						<!-- Form Name -->
						<xsl:variable name="FORMOID" select="@FormOID"/>
						<xsl:variable name="FORMNAME">
							<!--xsl:choose>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='FormDef'][@OID=$FORMOID]/@Name != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='FormDef'][@OID=$FORMOID]/@Name"/>
								</xsl:when>
								<xsl:otherwise><xsl:value-of select="$FORMOID"/></xsl:otherwise>
							</xsl:choose-->
							<!-- 3.3.2011 -->
							<xsl:call-template name="GetNameOfDefElement">
								<xsl:with-param name="studyoid" select="$STUDYOID"/>
								<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
								<xsl:with-param name="elementname">FormDef</xsl:with-param>
								<xsl:with-param name="elementoid" select="$FORMOID"/>
							</xsl:call-template>
						</xsl:variable>
						<xsl:attribute name="FormName"><xsl:value-of select="$FORMNAME"/></xsl:attribute>
						<!-- ItemGroup Name -->
						<xsl:variable name="ITEMGROUPOID" select="@ItemGroupOID"/>
						<xsl:variable name="ITEMGROUPNAME">
							<!--xsl:choose>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='ItemGroupDef'][@OID=$ITEMGROUPOID]/@Name != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='ItemGroupDef'][@OID=$ITEMGROUPOID]/@Name"/>
								</xsl:when>
								<xsl:otherwise><xsl:value-of select="$ITEMGROUPOID"/></xsl:otherwise>
							</xsl:choose-->
							<!-- 3.3.2011 -->
							<xsl:call-template name="GetNameOfDefElement">
								<xsl:with-param name="studyoid" select="$STUDYOID"/>
								<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
								<xsl:with-param name="elementname">ItemGroupDef</xsl:with-param>
								<xsl:with-param name="elementoid" select="$ITEMGROUPOID"/>
							</xsl:call-template>
						</xsl:variable>
						<xsl:attribute name="ItemGroupName"><xsl:value-of select="$ITEMGROUPNAME"/></xsl:attribute>
						<!-- Item Name -->
						<xsl:variable name="ITEMOID" select="@ItemOID"/>
						<xsl:variable name="ITEMNAME">
							<!--xsl:choose>
								// normal case, the Item Name can be found in the same MetaDataVersion 
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='ItemDef'][@OID=$ITEMOID]/@Name != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='ItemDef'][@OID=$ITEMOID]/@Name"/>
								</xsl:when>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='Include']">
									<xsl:variable name="INCLUDESTUDYOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='Include']/@StudyOID"/>
									<xsl:variable name="INCLUDEMETADATAVERSIONOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/*[name()='Include']/@MetaDataVersionOID"/>
									<xsl:message>Trying to find ItemName through Include, $INCLUDSTUDYOID = <xsl:value-of select="$INCLUDESTUDYOID"/> - $INCLUDEMETADATAVERSIONOID = <xsl:value-of select="$INCLUDEMETADATAVERSIONOID"/></xsl:message>
									<xsl:value-of select="'HALLO'"/>
								</xsl:when>
								// last case: it could not be found 
								<xsl:otherwise><xsl:value-of select="$ITEMOID"/></xsl:otherwise>
							</xsl:choose-->
							<!-- 3.3.2011 -->
							<xsl:call-template name="GetNameOfDefElement">
								<xsl:with-param name="studyoid" select="$STUDYOID"/>
								<xsl:with-param name="metadataversionoid" select="$METADATAVERSIONOID"/>
								<xsl:with-param name="elementname">ItemDef</xsl:with-param>
								<xsl:with-param name="elementoid" select="$ITEMOID"/>
							</xsl:call-template>
						</xsl:variable>
						<xsl:attribute name="ItemName"><xsl:value-of select="$ITEMNAME"/></xsl:attribute>
						<!-- child elements -->
						<!-- Auditor (is a User) -->
						<xsl:variable name="USEROID" select="./UserOID"/>
						<xsl:variable name="USERNAME">
							<xsl:choose>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='User'][@OID=$USEROID]/*[name()='FullName'] != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='User'][@OID=$USEROID]/*[name()='FullName']"/>
								</xsl:when>
								<xsl:otherwise><xsl:value-of select="$USEROID"/></xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:element name="UserName"><xsl:value-of select="$USERNAME"/></xsl:element>
						<!-- Auditor Location -->
						<xsl:variable name="LOCATIONOID" select="./LocationOID"></xsl:variable>
						<xsl:variable name="LOCATIONNAME">
							<xsl:choose>
								<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='Location'][@OID=$LOCATIONOID]/@Name != ''">
									<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$STUDYOID]/*[name()='Location'][@OID=$SITEOID]/@Name"/>
								</xsl:when>
								<xsl:otherwise><xsl:value-of select="$LOCATIONOID"/></xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:element name="LocationName"><xsl:value-of select="$LOCATIONNAME"/></xsl:element>
						
						<!-- copy the child elements -->
						<xsl:copy-of select="./*"/>
					</xsl:copy>
				</xsl:for-each>
			</xsl:copy>
		</xsl:when>
		
		<!-- in non-treated cases, do a deep copy -->
		<xsl:otherwise>
			<xsl:copy-of select="."></xsl:copy-of>
		</xsl:otherwise>
		</xsl:choose>
		
		
		<!--xsl:apply-templates/-->
	</xsl:template>
	
	<!-- template for protocol -->
	<xsl:template match="Protocol">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<!-- copy the protocol attributes (if any) -->
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- For ODM 1.3, copy the contents of the Description element -->
			<xsl:for-each select="Description">
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<!-- add StudyEventRef elements -->
			<xsl:for-each select="StudyEventRef">
				<!-- copy the StudyEventRef element and its attributes -->
				<xsl:copy> 
					<!-- In order to get it working from Java, 
					We need to define the new attribute BEFORE copying the others ! -->
					<xsl:variable name="STUDYEVENTOID" select="@StudyEventOID"/>
					<!--xsl:variable name="STUDYEVENTNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='StudyEventDef'][@OID=$STUDYEVENTOID]/@Name"/-->
					<!-- 2.3.2011 -->
					<xsl:variable name="STUDYEVENTNAME">
						<!--xsl:choose>
							<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='StudyEventDef'][@OID=$STUDYEVENTOID]">
								<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='StudyEventDef'][@OID=$STUDYEVENTOID]/@Name"/>
							</xsl:when>
							<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']">
								<xsl:variable name="INCLUDESTUDYOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@StudyOID"/>
								<xsl:variable name="INCLUDEMETADATAVERSIONOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@MetaDataVersionOID"/>
								<xsl:message>Trying to find StudyEventName through Include, $INCLUDESTUDYOID = <xsl:value-of select="$INCLUDESTUDYOID"/> - $INCLUDEMETADATAVERSIONOID = <xsl:value-of select="$INCLUDEMETADATAVERSIONOID"/></xsl:message>
								<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$INCLUDESTUDYOID]/*[name()='MetaDataVersion'][@OID=$INCLUDEMETADATAVERSIONOID]/*[name()='StudyEventDef'][@OID=$STUDYEVENTOID]/@Name"/>
							</xsl:when>
						</xsl:choose-->
						<!-- 2.3.2011: GetNameOfDefElement allows recursive searching through chains of 'Include' elements -->
						<xsl:call-template name="GetNameOfDefElement">
							<xsl:with-param name="studyoid" select="$studyoid"/>
							<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							<xsl:with-param name="elementname" select="'StudyEventDef'"/>
							<xsl:with-param name="elementoid" select="$STUDYEVENTOID"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:attribute name="StudyEventName"><xsl:value-of select="$STUDYEVENTNAME"/></xsl:attribute>
					<!-- copy the other attributes -->
					<xsl:for-each select="@*"><xsl:copy/></xsl:for-each> 
				</xsl:copy> 
			</xsl:for-each>
			<!-- For ODM 1.3.1, copy the Alias elements -->
			<xsl:for-each select="Alias">
				<xsl:copy-of select="."/>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
		
	<xsl:template match="MetaDataVersionRef">
		<xsl:variable name="METADATAVERSIONOID"><xsl:value-of select="@MetaDataVersionOID"/></xsl:variable>
		<xsl:variable name="STUDYOID"><xsl:value-of select="@StudyOID"/></xsl:variable>
		<!-- copy the element and the attributes -->
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<xsl:attribute name="MetaDataVersionName">
				<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$STUDYOID]/*[name()='MetaDataVersion'][@OID=$METADATAVERSIONOID]/@Name"/>
			</xsl:attribute>
		</xsl:copy>
	</xsl:template>
	
	<!-- Template for StudyEventDef -->
	<xsl:template match="StudyEventDef">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<!-- copy the attributes (if any) -->
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- For ODM 1.3, copy the contents of the Description element -->
			<xsl:for-each select="Description">
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<xsl:for-each select="FormRef">
				<!-- get the Form Name -->
				<xsl:variable name="FORMOID" select="@FormOID"/>
				<!--xsl:variable name="FORMNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='FormDef'][@OID=$FORMOID]/@Name"/-->			
				<!-- 2.3.2011 -->
				<xsl:variable name="FORMNAME">
					<!-- 2.3.2011: GetNameOfDefElement allows recursive searching through chain of 'Include's -->
					<xsl:call-template name="GetNameOfDefElement">
						<xsl:with-param name="studyoid" select="$studyoid"/>
						<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
						<xsl:with-param name="elementname" select="'FormDef'"/>
						<xsl:with-param name="elementoid" select="$FORMOID"/>
					</xsl:call-template>
				</xsl:variable>
				<!-- copy the FormRef element and its attributes -->
				<xsl:copy>	
				<!--xsl:element name="FormRef"-->				
					<xsl:attribute name="FormName"><xsl:value-of select="$FORMNAME"/></xsl:attribute>
					<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				</xsl:copy>
				<!--/xsl:element-->
			</xsl:for-each>
			<!-- For ODM 1.3.1, copy the Alias elements -->
			<xsl:for-each select="Alias">
				<xsl:copy-of select="."/>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
	
	<!-- Template for FormDef -->
	<xsl:template match="FormDef">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<!-- copy the attributes (if any) -->
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- For ODM 1.3, copy the contents of the Description element -->
			<xsl:for-each select="Description">
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<xsl:for-each select="ItemGroupRef">
				<!-- get the ItemGroup Name -->
				<xsl:variable name="ITEMGROUPOID" select="@ItemGroupOID"/>
				<!--xsl:variable name="ITEMGROUPNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemGroupDef'][@OID=$ITEMGROUPOID]/@Name"/-->			
				<!-- 2.3.2011-->
				<xsl:variable name="ITEMGROUPNAME">
					<!-- GetNameOfDefElement allows recursive searching through chain of 'Include's -->
					<xsl:call-template name="GetNameOfDefElement">
						<xsl:with-param name="studyoid" select="$studyoid"/>
						<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
						<xsl:with-param name="elementname" select="'ItemGroupDef'"/>
						<xsl:with-param name="elementoid" select="$ITEMGROUPOID"/>
					</xsl:call-template>
				</xsl:variable>
				<!-- copy the ItemGroupRef element and its attributes -->
				<xsl:copy>	
				<!--xsl:element name="ItemGroupRef"-->				
					<xsl:attribute name="ItemGroupName"><xsl:value-of select="$ITEMGROUPNAME"/></xsl:attribute>
					<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				</xsl:copy>
				<!--/xsl:element-->
			</xsl:for-each>
			<!-- 12.3.2007 : deep-copy any ArchiveLayout elements -->
			<xsl:for-each select="ArchiveLayout">
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<!-- For ODM 1.3.1, copy the Alias elements -->
			<xsl:for-each select="Alias">
				<xsl:copy-of select="."/>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
	
	<!-- Template for ItemGroupDef -->
	<xsl:template match="ItemGroupDef">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>

		<!-- copy the attributes (if any) -->
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- For ODM 1.3, copy the contents of the Description element -->
			<xsl:for-each select="Description">
				<xsl:copy-of select="."/>
			</xsl:for-each>
			<xsl:for-each select="ItemRef">
				<!-- get the Item Name -->
				<xsl:variable name="ITEMOID" select="@ItemOID"/>
				<!--xsl:variable name="ITEMNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemDef'][@OID=$ITEMOID]/@Name"/-->			
				<!-- 2.3.2011 - GetNameOfDefElement allows recursion -->
				<xsl:variable name="ITEMNAME">
					<xsl:call-template name="GetNameOfDefElement">
						<xsl:with-param name="studyoid" select="$studyoid"/>
						<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
						<xsl:with-param name="elementname" select="'ItemDef'"/>
						<xsl:with-param name="elementoid" select="$ITEMOID"/>
					</xsl:call-template>
				</xsl:variable>
				<!-- TODO: also look in "Included" MetaDataVersions -->
				<!--xsl:variable name="CODELISTOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemDef'][@OID=$ITEMOID]/*[name()='CodeListRef']/@CodeListOID"/-->
				<xsl:variable name="CODELISTOID">
					<xsl:call-template name="GetCodeListOIDFromItemOID">
						<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
						<xsl:with-param name="studyoid" select="$studyoid"/>
						<xsl:with-param name="itemoid" select="$ITEMOID"/>
					</xsl:call-template>
				</xsl:variable>
				<!-- copy the ItemRef element and its attributes -->
				<xsl:copy>	
				<!--xsl:element name="ItemRef"-->				
					<xsl:attribute name="ItemName"><xsl:value-of select="$ITEMNAME"/></xsl:attribute>
					<!-- 27.12.2010 -->
					<xsl:if test="$CODELISTOID ne ''">
						<xsl:attribute name="CodeListOID"><xsl:value-of select="$CODELISTOID"/></xsl:attribute>
					</xsl:if>
					<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
				</xsl:copy>
				<!--/xsl:element-->
			</xsl:for-each>
			<!-- For ODM 1.3.1, copy the Alias elements -->
			<xsl:for-each select="Alias">
				<xsl:copy-of select="."/>
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
	
	
	<!-- Template for ClinicalData -->
	<xsl:template match="ItemGroupData">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<!-- copy the element itself -->
		<xsl:copy>
			<!-- copy the attributes, if any -->		
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- Find the ItemGroupName from the OID, if nothing can be found, just take the ItemGroupOID -->
			<xsl:variable name="ITEMGROUPOID" select="@ItemGroupOID"/>
			<!--xsl:variable name="ITEMGROUPNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemGroupDef'][@OID=$ITEMGROUPOID]/@Name"/-->			
			<!-- 3.3.2011 GetNameOfDefElement allows recursion searching in chain of 'Include's -->
			<xsl:variable name="ITEMGROUPNAME">
				<xsl:call-template name="GetNameOfDefElement">
					<xsl:with-param name="studyoid" select="$studyoid"/>
					<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
					<xsl:with-param name="elementname">ItemGroupDef</xsl:with-param>
					<xsl:with-param name="elementoid" select="$ITEMGROUPOID"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:choose>		
				<xsl:when test="$ITEMGROUPNAME">
					<xsl:attribute name="ItemGroupName"><xsl:value-of select="$ITEMGROUPNAME"/></xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="ItemGroupName"><xsl:value-of select="$ITEMGROUPOID"/></xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			
			
			<xsl:apply-templates select="AuditRecord">
				<xsl:with-param name="studyoid" select="$studyoid"/>
				<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
			</xsl:apply-templates>
			<!-- treat any Signatures -->
			<xsl:apply-templates select="Signature">
				<xsl:with-param name="studyoid" select="$studyoid"/>
				<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
			</xsl:apply-templates>
			<!--xsl:copy-of select="Signature"/-->
			<!-- do a deep copy of Annotation -->
			<!--xsl:copy-of select="Annotation"/-->
			<xsl:apply-templates select="Annotation"/>
			<!-- copy the ItemData element and its attributes-->
			<!-- pre-ODM 1.3 
				<xsl:for-each select="ItemData">
			-->
			<!-- Extended for ODM 1.3.1 with ItemDataIncompleteDate and ItemDataIncompleteTime -->
			<xsl:for-each select="ItemData | ItemDataString | ItemDataInteger | ItemDataFloat | ItemDataDate | ItemDataTime 
				| ItemDataDatetime | ItemDataBoolean | ItemDataDouble | ItemDataHexBinary | ItemDataBase64Binary  | ItemDataHexFloat 
				| ItemDataBase64Float | ItemDataPartialDate | ItemDataPartialTime | ItemDataPartialDatetime | ItemDataDurationDatetime 
				| ItemDataIntervalDatetime | ItemDataIncompleteDatetime | ItemDataIncompleteDate | ItemDataIncompleteTime | ItemDataURI | ItemDataAny">
				<!--xsl:message>COPYING <xsl:value-of select="."/></xsl:message-->
				<!-- copy the element and its attributes -->
				<xsl:copy>
					<xsl:for-each select="@*"><xsl:copy/></xsl:for-each> <!-- copy the existing attributes -->
					<!-- find the Item Name from the OID, if nothing found, just take the ItemOID -->
					<!-- Also check for a codelist, and decode the value if possible -->
					<xsl:variable name="ITEMOID"><xsl:value-of select="@ItemOID"/></xsl:variable>
					<xsl:variable name="VALUE">
						<xsl:choose>
							<!-- ODM 1.2 untyped ItemData -->
							<xsl:when test="name() = 'ItemData'"><xsl:value-of select="@Value"/></xsl:when>
							<!-- ODM 1.3 typed ItemData -->
							<xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					<!--xsl:variable name="ITEMNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemDef'][@OID=$ITEMOID]/@Name"/-->
					<!-- 3.3.2011-->
					<xsl:variable name="ITEMNAME">
						<xsl:call-template name="GetNameOfDefElement">
							<xsl:with-param name="studyoid" select="$studyoid"/>
							<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							<xsl:with-param name="elementname">ItemDef</xsl:with-param>
							<xsl:with-param name="elementoid" select="$ITEMOID"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:choose>
						<xsl:when test="$ITEMNAME">
							<xsl:attribute name="ItemName"><xsl:value-of select="$ITEMNAME"/></xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="ItemName"><xsl:value-of select="$ITEMOID"/></xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<!-- if there is a codelist, transform the name -->
					<xsl:variable name="CODELIST" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemDef'][@OID=$ITEMOID]/*[name()='CodeListRef']/@CodeListOID"/>
					<!-- TODO : different languages in TranslatedText (xml:lang attribute -->
					<xsl:variable name="REALVALUE">
						<xsl:choose>
							<xsl:when test="$CODELIST">
								<xsl:variable name="DECODEDVALUE" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='CodeList'][@OID=$CODELIST]/*[name()='CodeListItem'][@CodedValue=$VALUE]/*[name()='Decode']/*[name()='TranslatedText']"/>
								<!-- replace the value of the Value attribute -->
								<xsl:choose>
								<xsl:when test="$DECODEDVALUE"> <!-- the coded value is in the list, apply it -->
									<xsl:value-of select="$DECODEDVALUE"/>
								</xsl:when>
								<xsl:otherwise> <!-- the value is NOT in the list - DECODE ERROR ! -->
									<!--xsl:value-of select="$CODELISTERROR"/-->
									<!--xsl:value-of select="concat($VALUE,' ',$CODELISTERROR)"/--><!-- Behring 8.7.2014 -->
									<xsl:if test="normalize-space($VALUE) != '' "><!-- CSL Behring 25.8.2014 for Value='' (essentially ODM non-conform) -->
										<xsl:value-of select="concat($VALUE,' ',$CODELISTERROR)"/><!-- CSL Behring 8.7.2014 -->
									</xsl:if>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise><xsl:value-of select="$VALUE"/></xsl:otherwise>
					</xsl:choose>
					</xsl:variable>
					<xsl:attribute name="RealValue"><xsl:value-of select="$REALVALUE"/></xsl:attribute>
					<!-- treat any AuditRecord -->
					<xsl:apply-templates select="AuditRecord">
						<xsl:with-param name="studyoid" select="$studyoid"/>
						<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
					</xsl:apply-templates>
					<!-- treat any signatures -->
					<xsl:apply-templates select="Signature">
						<xsl:with-param name="studyoid" select="$studyoid"/>
						<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
					</xsl:apply-templates>
					<!-- TODO: MeasurementUnitOID -> MeasurementUnitName -->
					<!--xsl:copy-of select="MeasurementUnitRef"/-->
					<xsl:apply-templates select="MeasurementUnitRef">
						<xsl:with-param name="studyoid" select="$studyoid"/>
						<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
					</xsl:apply-templates>
					<!--
					<xsl:for-each select="MeasurementUnitRef">
						<xsl:variable name="MEASUREMENTUNITOID" select="@MeasurementUnitOID"/>
						<xsl:copy>
							<xsl:copy-of select="@*"/>
							<xsl:attribute name="MeasurementUnitName">
								<xsl:choose>
									<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='BasicDefinitions']/*[name()='MeasurementUnit'][@OID=$MEASUREMENTUNITOID]">
										<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='BasicDefinitions']/*[name()='MeasurementUnit'][@OID=$MEASUREMENTUNITOID]/@Name"/>
									</xsl:when>
									<xsl:otherwise><xsl:value-of select="$MEASUREMENTUNITOID"/></xsl:otherwise>
								</xsl:choose>
							</xsl:attribute>
						</xsl:copy>
					</xsl:for-each> -->
					<!-- do a deep copy of Annotation -->
					<!--xsl:copy-of select="Annotation"/-->
					<xsl:apply-templates select="Annotation"/>
				</xsl:copy>
			</xsl:for-each>
			<!-- deep-copy the AdditionalInfo -->
			<!--xsl:copy-of select="AdditionalInfo"/-->
			<xsl:for-each select="AdditionalInfo">
				<xsl:copy><!-- copy the AdditionalInfo element itself -->
					<!-- important change 11.9.2014 - we need to copy child element by child element!
						Do not forget anything! -->
					<!--xsl:copy-of select="*"/-->
					<xsl:copy-of select="SubjectKey"/>
					<xsl:copy-of select="StudyEventOID"/>
					<!--xsl:copy-of select="StudyEventRepeatKey"/-->
					<xsl:copy-of select="FormOID"/>
					<!--xsl:copy-of select="FormRepeatKey"/-->
					<xsl:copy-of select="InvestigatorOID"/>
					<xsl:copy-of select="SiteOID"/>
					<!-- try to find the StudyEvent Name from the StudyEventOID, if nothing is found, use the StudyEventOID -->
					<xsl:variable name="STUDYEVENTOID"><xsl:value-of select="StudyEventOID"/></xsl:variable>
					<xsl:variable name="FORMOID"><xsl:value-of select="FormOID"/></xsl:variable>
					<!--xsl:variable name="STUDYEVENTNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='StudyEventDef'][@OID=$STUDYEVENTOID]/@Name"/-->	
					<!-- 3.3.2011 -->
					<xsl:variable name="STUDYEVENTNAME">
						<xsl:call-template name="GetNameOfDefElement">
							<xsl:with-param name="studyoid" select="$studyoid"/>
							<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							<xsl:with-param name="elementname">StudyEventDef</xsl:with-param>
							<xsl:with-param name="elementoid" select="$STUDYEVENTOID"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:variable name="INVESTIGATOROID" select="InvestigatorOID"/>
					<xsl:variable name="SITEOID" select="SiteOID"/>
					<xsl:element name="StudyEventName">
						<xsl:choose>
							<xsl:when test="$STUDYEVENTNAME"><xsl:value-of select="$STUDYEVENTNAME"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$STUDYEVENTOID"/></xsl:otherwise>
						</xsl:choose>
					</xsl:element>		
					<!-- try to find the Form Name from the FormOID, if nothing is found, use the FormOID -->
					<!--xsl:variable name="FORMNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='FormDef'][@OID=$FORMOID]/@Name"/-->	
					<!-- 3.3.2011 -->
					<xsl:variable name="FORMNAME">
						<xsl:call-template name="GetNameOfDefElement">
							<xsl:with-param name="studyoid" select="$studyoid"/>
							<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							<xsl:with-param name="elementname">FormDef</xsl:with-param>
							<xsl:with-param name="elementoid" select="$FORMOID"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:element name="FormName">
						<xsl:choose>
							<xsl:when test="$FORMNAME"><xsl:value-of select="$FORMNAME"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$FORMOID"/></xsl:otherwise>
						</xsl:choose>
					</xsl:element>
					<!-- 5.6.2009: add Investigator and Location information -->
					<xsl:variable name="INVESTIGATORNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$studyoid]/*[name()='User'][@OID=$INVESTIGATOROID]/*[name()='FullName']"/>
					<!--xsl:message>INVESTIGATOROID = <xsl:value-of select="$INVESTIGATOROID"/> - INVESTIGATORNAME = <xsl:value-of select="$INVESTIGATORNAME"/></xsl:message-->
					<xsl:element name="InvestigatorName">
						<xsl:choose>
							<xsl:when test="$INVESTIGATORNAME"><xsl:value-of select="$INVESTIGATORNAME"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$INVESTIGATOROID"/></xsl:otherwise>
						</xsl:choose>
					</xsl:element>
					<xsl:variable name="SITENAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$studyoid]/*[name()='Location'][@OID=$SITEOID]/@Name"/>
					<xsl:element name="SiteName">
						<xsl:choose>
							<xsl:when test="$SITENAME"><xsl:value-of select="$SITENAME"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$SITEOID"/></xsl:otherwise>
						</xsl:choose>
					</xsl:element>
					<!-- 11.9.2014 Upper level Signatures, AuditRecords and Annotations (for inheritence) -->
					<xsl:for-each select="SubjectDataAuditRecord">
						<xsl:copy>
							<xsl:apply-templates select="AuditRecord">
								<xsl:with-param name="studyoid" select="$studyoid"/>
								<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							</xsl:apply-templates>
						</xsl:copy>
					</xsl:for-each>
					<xsl:for-each select="StudyEventDataAuditRecord">
						<xsl:copy>
							<xsl:apply-templates select="AuditRecord">
								<xsl:with-param name="studyoid" select="$studyoid"/>
								<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							</xsl:apply-templates>
						</xsl:copy>
					</xsl:for-each>
					<xsl:for-each select="FormDataAuditRecord">
						<xsl:copy>
							<xsl:apply-templates select="AuditRecord">
								<xsl:with-param name="studyoid" select="$studyoid"/>
								<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							</xsl:apply-templates>
						</xsl:copy>
					</xsl:for-each>
					<!-- and now the same for inheritence of Signatures -->
					<xsl:for-each select="SubjectDataSignature">
						<xsl:copy>
							<xsl:apply-templates select="Signature">
								<xsl:with-param name="studyoid" select="$studyoid"/>
								<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							</xsl:apply-templates>
						</xsl:copy>
					</xsl:for-each>
					<xsl:for-each select="StudyEventDataSignature">
						<xsl:copy>
							<xsl:apply-templates select="Signature">
								<xsl:with-param name="studyoid" select="$studyoid"/>
								<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							</xsl:apply-templates>
						</xsl:copy>
					</xsl:for-each>
					<xsl:for-each select="FormDataSignature">
						<xsl:copy>
							<xsl:apply-templates select="Signature">
								<xsl:with-param name="studyoid" select="$studyoid"/>
								<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
							</xsl:apply-templates>
						</xsl:copy>
					</xsl:for-each>
					<!-- copy the SubjectDataAnnotions, StudyEventDataAnnotations, FormDataAnnotations without any change -->
					<xsl:copy-of select="SubjectDataAnnotations"/>
					<xsl:copy-of select="StudyEventDataAnnotations"/>
					<xsl:copy-of select="FormDataAnnotations"/>
				</xsl:copy><!-- end of copy the AdditionalInfo element itself -->
			</xsl:for-each>
		</xsl:copy>
	</xsl:template>
	
	<!-- adds an attribute CodeListName from the CodeListRef CodeListOID -->
	<xsl:template match="CodeListRef">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<xsl:variable name="CODELISTOID" select="@CodeListOID"/>
		<xsl:copy> <!-- copy the CodeListRef element and its attributes -->
			<xsl:copy-of select="@*"/>
			<xsl:variable name="CODELISTNAME">
				<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='CodeList'][@OID=$CODELISTOID]/@Name"/>
			</xsl:variable>
			<xsl:attribute name="CodeListName"><xsl:value-of select="$CODELISTNAME"/></xsl:attribute>
		</xsl:copy>
	</xsl:template>
	
	<!-- adds MeasurementUnit Name to the MeasurementUnitRef -->
	<xsl:template match="MeasurementUnitRef">
		<xsl:param name="studyoid"></xsl:param>
		<xsl:param name="metadataversionoid"></xsl:param>
		<xsl:variable name="MEASUREMENTUNITOID" select="@MeasurementUnitOID"/>
		<xsl:copy>
			<xsl:copy-of select="@*"/>
			<xsl:attribute name="MeasurementUnitName">
				<xsl:choose>
					<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='BasicDefinitions']/*[name()='MeasurementUnit'][@OID=$MEASUREMENTUNITOID]">
						<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='BasicDefinitions']/*[name()='MeasurementUnit'][@OID=$MEASUREMENTUNITOID]/@Name"/>
					</xsl:when>
					<xsl:otherwise><xsl:value-of select="$MEASUREMENTUNITOID"/></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="AuditRecord">
		<xsl:param name="studyoid"/>
		<xsl:param name="metadataversionoid"/>
		<!--xsl:message>Running AuditRecord template with parameters:</xsl:message>
		<xsl:message>studyoid = <xsl:value-of select="$studyoid"/></xsl:message>
		<xsl:message>metadataversionoid = <xsl:value-of select="$metadataversionoid"/></xsl:message-->
		<!-- copy the element itself and its attributes -->
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- UserRef -->
			<xsl:apply-templates select="UserRef">
				<xsl:with-param name="studyoid" select="$studyoid"/>
			</xsl:apply-templates>
			<!-- LocationRef -->
			<xsl:apply-templates select="LocationRef">
				<xsl:with-param name="studyoid" select="$studyoid"/>
				<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
			</xsl:apply-templates>
			<!-- do a deep copy of DateTimeStamp -->
			<xsl:copy-of select="DateTimeStamp"/>
			<!-- do a deep copy of ReasonForChange -->
			<xsl:copy-of select="ReasonForChange"/>
			<!-- do a deep copy of SourceID -->
			<xsl:copy-of select="SourceID"/>
		</xsl:copy>
	</xsl:template>
	
	<!-- template for Signature (not ds:Signature)
		Signature has following subelements:
		UserRef, LocationRef, SignatureRef, DateTimeStamp (each exactly 1 occurrence), and
		CryptoBindingManifest (0 or 1) -->
	<xsl:template match="Signature">
		<xsl:param name="studyoid"/>
		<xsl:param name="metadataversionoid"/>
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- UserRef -->
			<xsl:apply-templates select="UserRef">
				<xsl:with-param name="studyoid" select="$studyoid"/>
			</xsl:apply-templates>
			<!-- LocationRef -->
			<xsl:apply-templates select="LocationRef">
				<xsl:with-param name="studyoid" select="$studyoid"/>
				<xsl:with-param name="metadataversionoid" select="$metadataversionoid"/>
			</xsl:apply-templates>
			<!-- SignatureRef -->
			<xsl:apply-templates select="SignatureRef">
				<xsl:with-param name="studyoid" select="$studyoid"/>
			</xsl:apply-templates>
			<!-- do a deep copy of DateTimeStamp -->
			<xsl:copy-of select="DateTimeStamp"/>
			<!-- do a deep copy of CryptoBindingManifest -->
			<xsl:copy-of select="CryptoBindingManifest"/>
		</xsl:copy>
	</xsl:template>
	
	<!-- find the location from the LocationRef LocationOID -->
	<xsl:template match="LocationRef">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<xsl:variable name="LOCATIONOID" select="@LocationOID"/>
		<!--xsl:message>Got LOCATIONOID = <xsl:value-of select="$LOCATIONOID"/></xsl:message>
		<xsl:message>with STUDYOID = <xsl:value-of select="$studyoid"/></xsl:message-->
		
		<xsl:copy>
			<xsl:copy-of select="@*"/>
			<xsl:variable name="LOCATIONNAME">
				<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$studyoid]/*[name()='Location'][@OID=$LOCATIONOID]/@Name"/>
			</xsl:variable>
			<!--xsl:message>LOCATIONNAME = <xsl:value-of select="$LOCATIONNAME"/></xsl:message-->
			<xsl:attribute name="LocationName">
				<xsl:choose>
					<xsl:when test="$LOCATIONNAME">
						<xsl:value-of select="$LOCATIONNAME"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$LOCATIONOID"/> <!-- in case the data is not available -->
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</xsl:copy>
	</xsl:template>
	
	<!-- find the user name from the UserOID -->
	<xsl:template match="UserRef">
		<xsl:param name="studyoid"/>
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>					
			<!-- add the user name if available from the UserOID, if nothing found, take UserOID as UserName -->
			<xsl:variable name="USEROID" select="@UserOID"/>
			<xsl:variable name="USERNAME" select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$studyoid]/*[name()='User'][@OID=$USEROID]/*[name()='FullName']"/>	
			<xsl:choose>
				<xsl:when test="$USERNAME">
					<xsl:attribute name="UserName"><xsl:value-of select="$USERNAME"/></xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="UserName"><xsl:value-of select="$USEROID"/></xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:copy>
	</xsl:template>
	
	<!-- SignatureRef: add information from SignatureDef: Methodology (attribute), Meaning (subelement), LegalReason.
		Add these as attributes -->
	<xsl:template match="SignatureRef">
		<xsl:param name="studyoid"/>
		<!-- copy the element itself and its attributes -->
		<xsl:copy>
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- find the necessary information in SignatureDef in AdminData -->
			<xsl:variable name="SIGNATUREIOID" select="@SignatureOID"/>
			<!-- find in AdminData -->
			<xsl:variable name="METHODOLOGY" select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$studyoid]/*[name()='SignatureDef'][@OID=$SIGNATUREIOID]/@Methodology"/>
			<xsl:variable name="MEANING" select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$studyoid]/*[name()='SignatureDef'][@OID=$SIGNATUREIOID]/*[name()='Meaning']"/>
			<xsl:variable name="LEGALREASON" select="document($ODMDOC)/*[name()='ODM']/*[name()='AdminData'][@StudyOID=$studyoid]/*[name()='SignatureDef'][@OID=$SIGNATUREIOID]/*[name()='LegalReason']"/>
			<xsl:attribute name="Methodology"><xsl:value-of select="$METHODOLOGY"/></xsl:attribute>
			<xsl:attribute name="Meaning"><xsl:value-of select="$MEANING"/></xsl:attribute>
			<xsl:attribute name="LegalReason"><xsl:value-of select="$LEGALREASON"/></xsl:attribute>
		</xsl:copy>
	</xsl:template>
	
	<!-- 18.6.2009 Annotation - supply CodeListName on CodeList child of FlagValue -->
	<xsl:template match="Annotation">
		<xsl:copy>
			<!-- copy the attributes -->
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- copy the content of the "Comment" child element -->
			<xsl:copy-of select="Comment"/>
			<!-- and continue working on the "Flag" child elements -->
			<xsl:apply-templates select="Flag"/>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="Flag">
		<!-- Child elements: FlagValue (exactly 1), FlagType (0 or 1) -->
		<xsl:copy>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="FlagValue">
		<xsl:copy>
			<!-- FlagValue has CodeListOID attribute and text content -->
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- and add a "CodeListName" attribute -->
			<xsl:variable name="CODELISTOID" select="@CodeListOID"/>
			<!-- we need to find the StudyOID and MetaDataVersionOID, but this depends on where in the tree we are -->
			<xsl:variable name="ANNOTATIONPARENT" select="../../../name()"/>
			<!--xsl:message>ANNOTATIONPARENT = <xsl:value-of select="$ANNOTATIONPARENT"/></xsl:message-->
			<!-- For getting the StudyOID and MetaDataVersionOID - we need to get up to the "Results" element -->
			<xsl:variable name="studyoid">
				<xsl:choose>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemData'"><xsl:value-of select="../../../../../@StudyOID"/></xsl:when>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemGroupData'"><xsl:value-of select="../../../../@StudyOID"/></xsl:when>
					<!-- ODM 1.3 - still needs testing -->
					<xsl:when test="$ANNOTATIONPARENT = 'Annotations'"><xsl:value-of select="../../@StudyOID"/></xsl:when>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="metadataversionoid">
				<xsl:choose>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemData'"><xsl:value-of select="../../../../../@MetaDataVersionOID"/></xsl:when>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemGroupData'"><xsl:value-of select="../../../../@MetaDataVersionOID"/></xsl:when>
					<!-- ODM 1.3 - still needs testing -->
					<xsl:when test="$ANNOTATIONPARENT = 'Annotations'"><xsl:value-of select="../../@MetaDataVersionOID"/></xsl:when>
				</xsl:choose>
			</xsl:variable>
			<!--xsl:message>$studyOID = <xsl:value-of select="$studyoid"/></xsl:message>
			<xsl:message>$metadataversionoid = <xsl:value-of select="$metadataversionoid"/></xsl:message-->
			<xsl:variable name="CODELISTNAME">
				<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='CodeList'][@OID=$CODELISTOID]/@Name"/>
			</xsl:variable>
			<xsl:attribute name="CodeListName">
				<xsl:choose>
					<xsl:when test="$CODELISTNAME != ''"><xsl:value-of select="$CODELISTNAME"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="$CODELISTOID"/></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<!-- and copy the text contents -->
			<xsl:copy-of select="text()"/>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="FlagType">
		<xsl:copy>
			<!-- FlagType has CodeListOID attribute and text content -->
			<xsl:for-each select="@*"><xsl:copy/></xsl:for-each>
			<!-- add a CodeListName attribute -->
			<!-- and add a "CodeListName" attribute -->
			<xsl:variable name="CODELISTOID" select="@CodeListOID"/>
			<!-- we need to find the StudyOID and MetaDataVersionOID, but this depends on where in the tree we are -->
			<xsl:variable name="ANNOTATIONPARENT" select="../../../name()"/>
			<!--xsl:message>ANNOTATIONPARENT = <xsl:value-of select="$ANNOTATIONPARENT"/></xsl:message-->
			<!-- For getting the StudyOID and MetaDataVersionOID - we need to get up to the "Results" element -->
			<xsl:variable name="studyoid">
				<xsl:choose>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemData'"><xsl:value-of select="../../../../../@StudyOID"/></xsl:when>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemGroupData'"><xsl:value-of select="../../../../@StudyOID"/></xsl:when>
					<!-- ODM 1.3 - still needs testing -->
					<xsl:when test="$ANNOTATIONPARENT = 'Annotations'"><xsl:value-of select="../../@StudyOID"/></xsl:when>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="metadataversionoid">
				<xsl:choose>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemData'"><xsl:value-of select="../../../../../@MetaDataVersionOID"/></xsl:when>
					<xsl:when test="$ANNOTATIONPARENT = 'ItemGroupData'"><xsl:value-of select="../../../../@MetaDataVersionOID"/></xsl:when>
					<!-- ODM 1.3 - still needs testing -->
					<xsl:when test="$ANNOTATIONPARENT = 'Annotations'"><xsl:value-of select="../../@MetaDataVersionOID"/></xsl:when>
				</xsl:choose>
			</xsl:variable>
			<!--xsl:message>$studyOID = <xsl:value-of select="$studyoid"/></xsl:message>
			<xsl:message>$metadataversionoid = <xsl:value-of select="$metadataversionoid"/></xsl:message-->
			<xsl:variable name="CODELISTNAME">
				<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='CodeList'][@OID=$CODELISTOID]/@Name"/>
			</xsl:variable>
			<xsl:attribute name="CodeListName">
				<xsl:choose>
					<xsl:when test="$CODELISTNAME != ''"><xsl:value-of select="$CODELISTNAME"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="$CODELISTOID"/></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<!-- and copy the text contents -->
			<xsl:copy-of select="text()"/>
		</xsl:copy>
	</xsl:template>
	
	<!-- special stuff -->
	<!-- 2.3.2011-->
	<!-- this looks for the value of the @Name attribute for an XXXDef element in a specific MetaDataVersion/Study. 
		If not found, it tries to find an 'Include' and recursively looks into the MetaDataVersion/Study to be included -->
	<xsl:template name="GetNameOfDefElement">
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="studyoid"/>
		<xsl:param name="elementname"/>
		<xsl:param name="elementoid"/>
		<xsl:choose>
			<!-- first case: XXXDef in the same MetaDataVersion -->
			<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()=$elementname][@OID=$elementoid]">
				<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()=$elementname][@OID=$elementoid]/@Name"/>
			</xsl:when>
			<!-- second case: Not in the same MetaDataVersion, but there is an Include -->
			<!-- works recursively -->
			<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']">
				<xsl:variable name="INCLUDESTUDYOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@StudyOID"/>
				<xsl:variable name="INCLUDEMETADATAVERSIONOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@MetaDataVersionOID"/>
				<!--xsl:message>Trying to find Name for element <xsl:value-of select="$elementname"/> with OID <xsl:value-of select="$elementoid"/> through Include, $INCLUDESTUDYOID = <xsl:value-of select="$INCLUDESTUDYOID"/> - $INCLUDEMETADATAVERSIONOID = <xsl:value-of select="$INCLUDEMETADATAVERSIONOID"/></xsl:message-->
				<!-- implements recursion -->
				<xsl:variable name="NAMEVALUE">
					<xsl:choose>
						<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$INCLUDESTUDYOID]/*[name()='MetaDataVersion'][@OID=$INCLUDEMETADATAVERSIONOID]/*[name()=$elementname][@OID=$elementoid]">
							<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$INCLUDESTUDYOID]/*[name()='MetaDataVersion'][@OID=$INCLUDEMETADATAVERSIONOID]/*[name()=$elementname][@OID=$elementoid]/@Name"/>
						</xsl:when>
						<!-- nothing found in the to-be-included MetaDataVersion, try recursion -->
						<xsl:otherwise>
							<xsl:call-template name="GetNameOfDefElement">
								<xsl:with-param name="metadataversionoid" select="$INCLUDEMETADATAVERSIONOID"/>
								<xsl:with-param name="studyoid" select="$INCLUDESTUDYOID"/>
								<xsl:with-param name="elementname" select="$elementname"/>
								<xsl:with-param name="elementoid" select="$elementoid"/>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:value-of select="$NAMEVALUE"/>
			</xsl:when>
			<!-- no Include and not in this MetaDataVersion, return the OID -->
			<xsl:otherwise><xsl:value-of select="$elementoid"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Recursively searches the associated CodeList OID through a chain of 'Include's -->
	<xsl:template name="GetCodeListOIDFromItemOID">
		<xsl:param name="studyoid"/>
		<xsl:param name="metadataversionoid"/>
		<xsl:param name="itemoid"/>
		<!--xsl:message>Looking for CodeList for Item with OID = <xsl:value-of select="$itemoid"/></xsl:message-->
		<xsl:variable name="CODELISTOID">
			<xsl:choose>
				<!-- first case: the ItemDef is in the current MetaDataVersion/Study -->
				<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemDef'][@OID=$itemoid]">
					<xsl:value-of select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='ItemDef'][@OID=$itemoid]/*[name()='CodeListRef']/@CodeListOID"/>
				</xsl:when>
				<!-- nothing found, but there is an 'Include' statement -->
				<xsl:when test="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']">
					<!-- get the to-be-included StudyOID and MetaDataVersionOID -->
					<xsl:variable name="INCLUDESTUDYOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@StudyOID"/>
					<xsl:variable name="INCLUDEMETADATAVERSIONOID" select="document($ODMDOC)/*[name()='ODM']/*[name()='Study'][@OID=$studyoid]/*[name()='MetaDataVersion'][@OID=$metadataversionoid]/*[name()='Include']/@MetaDataVersionOID"/>
					<!-- RECURSION -->
					<xsl:call-template name="GetCodeListOIDFromItemOID">
						<xsl:with-param name="studyoid" select="$INCLUDESTUDYOID"/>
						<xsl:with-param name="metadataversionoid" select="$INCLUDEMETADATAVERSIONOID"/>
						<xsl:with-param name="itemoid" select="$itemoid"/>
					</xsl:call-template>
				</xsl:when>
				<!-- not found in the current MetaDataVersion/Study and there is also no Include statement -->
				<xsl:otherwise></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!--xsl:message>$CODELISTOID = <xsl:value-of select="$CODELISTOID"/></xsl:message-->
		<xsl:value-of select="$CODELISTOID"/>
		
	</xsl:template>
	
	
</xsl:stylesheet>